/*
 * An object for a "host" directory provided via the C library.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <dirent.h>
#include <stdlib.h>
#include <string.h>

#include <fsclient/client.h>

#include "host_directory_accessor.h"



/* Initialise the accessor, copying the path. */

HostDirectoryAccessor::HostDirectoryAccessor(const char *path)
{
    _path = strdup(path);
}

/* Release the copied path upon deletion. */

HostDirectoryAccessor::~HostDirectoryAccessor()
{
    if (_path != NULL)
        free(_path);
}

/* Write directory entries to the given 'writer'. */

void HostDirectoryAccessor::read_directory(file_t *writer)
{
    DIR *dir = opendir(_path);
    struct dirent *dirent;

    /* Write directory entries to the pipe, closing the pipe when finished. */

    while ((dirent = readdir(dir)) != NULL)
    {
        offset_t nwritten = client_write(writer, (const void *) dirent, dirent->d_reclen);

        /* Stop writing if the pipe is closed. */

        if (nwritten < dirent->d_reclen)
            break;
    }
}

// vim: tabstop=4 expandtab shiftwidth=4
